@@ -283,8 +283,7 @@ gdk_pixbuf__png_image_load (FILE *f, GError **error)
 	}
 
 	if (setjmp (png_ptr->jmpbuf)) {
-	    	if (rows)
-		  	g_free (rows);
+	    	g_free (rows);
 
 		if (pixbuf)
 			g_object_unref (pixbuf);
@@ -344,8 +343,8 @@ gdk_pixbuf__png_image_load (FILE *f, GError **error)
 static void png_error_callback  (png_structp png_read_ptr,
                                  png_const_charp error_msg);
 
-static void png_warning_callback(png_structp png_read_ptr,
-                                 png_const_charp warning_msg);
+static void png_warning_callback (png_structp png_read_ptr,
+                                  png_const_charp warning_msg);
 
 /* Called at the start of the progressive load */
 static void png_info_callback   (png_structp png_read_ptr,
@@ -676,7 +675,7 @@ png_row_callback   (png_structp png_read_ptr,
         if (lc->fatal_error_occurred)
                 return;
 
-        if (row_num < 0 || row_num >= lc->pixbuf->height) {
+        if (row_num >= lc->pixbuf->height) {
                 lc->fatal_error_occurred = TRUE;
                 if (lc->error && *lc->error == NULL) {
                         g_set_error (lc->error,
@@ -739,8 +738,8 @@ png_error_callback(png_structp png_read_ptr,
 }
 
 static void
-png_warning_callback(png_structp png_read_ptr,
-                     png_const_charp warning_msg)
+png_warning_callback (png_structp png_read_ptr,
+                      png_const_charp warning_msg)
 {
         LoadContext* lc;
         
@@ -861,9 +860,7 @@ static gboolean real_save_png (GdkPixbuf        *pixbuf,
                                        return FALSE;
                                }
                        } else {
-                               g_warning ("Bad option name '%s' passed to PNG saver",
-                                          *kiter);
-                               return FALSE;
+                               g_warning ("Unrecognized parameter (%s) passed to PNG saver.", *kiter);
                        }
 
                        ++kiter;
@@ -1012,8 +1009,13 @@ gdk_pixbuf__png_image_save_to_callback (GdkPixbufSaveFunc   save_func,
                               TRUE, NULL, save_func, user_data);
 }
 
-void
-MODULE_ENTRY (png, fill_vtable) (GdkPixbufModule *module)
+#ifndef INCLUDE_png
+#define MODULE_ENTRY(function) G_MODULE_EXPORT void function
+#else
+#define MODULE_ENTRY(function) void _gdk_pixbuf__png_ ## function
+#endif
+
+MODULE_ENTRY (fill_vtable) (GdkPixbufModule *module)
 {
         module->load = gdk_pixbuf__png_image_load;
         module->begin_load = gdk_pixbuf__png_image_begin_load;
@@ -1023,8 +1025,7 @@ MODULE_ENTRY (png, fill_vtable) (GdkPixbufModule *module)
         module->save_to_callback = gdk_pixbuf__png_image_save_to_callback;
 }
 
-void
-MODULE_ENTRY (png, fill_info) (GdkPixbufFormat *info)
+MODULE_ENTRY (fill_info) (GdkPixbufFormat *info)
 {
         static GdkPixbufModulePattern signature[] = {
                 { "\x89PNG\r\n\x1a\x0a", NULL, 100 },
