@@ -21,9 +21,13 @@
  * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_MENU_SHELL_H__
 #define __GTK_MENU_SHELL_H__
 
@@ -32,10 +36,7 @@
 #include <gtk/gtkcontainer.h>
 
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 #define	GTK_TYPE_MENU_SHELL		(gtk_menu_shell_get_type ())
 #define GTK_MENU_SHELL(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_MENU_SHELL, GtkMenuShell))
@@ -63,7 +64,7 @@ struct _GtkMenuShell
   guint have_grab : 1;
   guint have_xgrab : 1;
   guint ignore_leave : 1;	/* unused */
-  guint menu_flag : 1;
+  guint menu_flag : 1;		/* unused */
   guint ignore_enter : 1;
 };
 
@@ -87,11 +88,12 @@ struct _GtkMenuShellClass
 			    GtkWidget    *child,
 			    gint          position);
   gint (*get_popup_delay)  (GtkMenuShell *menu_shell);
+  gboolean (*move_selected) (GtkMenuShell *menu_shell,
+			     gint          distance);
 
   /* Padding for future expansion */
   void (*_gtk_reserved1) (void);
   void (*_gtk_reserved2) (void);
-  void (*_gtk_reserved3) (void);
 };
 
 
@@ -116,10 +118,19 @@ void _gtk_menu_shell_select_last       (GtkMenuShell *menu_shell,
 					gboolean      search_sensitive);
 void  _gtk_menu_shell_activate         (GtkMenuShell *menu_shell);
 gint  _gtk_menu_shell_get_popup_delay  (GtkMenuShell *menu_shell);
+void  gtk_menu_shell_cancel            (GtkMenuShell *menu_shell);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+void  _gtk_menu_shell_add_mnemonic     (GtkMenuShell *menu_shell,
+                                        guint         keyval,
+                                        GtkWidget    *target);
+void  _gtk_menu_shell_remove_mnemonic  (GtkMenuShell *menu_shell,
+                                        guint         keyval,
+                                        GtkWidget    *target);
 
+gboolean gtk_menu_shell_get_take_focus (GtkMenuShell *menu_shell);
+void     gtk_menu_shell_set_take_focus (GtkMenuShell *menu_shell,
+                                        gboolean      take_focus);
+
+G_END_DECLS
 
 #endif /* __GTK_MENU_SHELL_H__ */
