@@ -28,7 +28,6 @@
 #include "gtkintl.h"
 #include "gtkpaned.h"
 #include "gtkbindings.h"
-#include "gtksignal.h"
 #include "gdk/gdkkeysyms.h"
 #include "gtkwindow.h"
 #include "gtkmain.h"
@@ -478,6 +477,9 @@ gtk_paned_class_init (GtkPanedClass *class)
   gtk_binding_entry_add_signal (binding_set,
 				GDK_Return, 0,
 				"accept_position", 0);
+  gtk_binding_entry_add_signal (binding_set,
+				GDK_ISO_Enter, 0,
+				"accept_position", 0);
   gtk_binding_entry_add_signal (binding_set,
 				GDK_KP_Enter, 0,
 				"accept_position", 0);
@@ -1235,7 +1237,10 @@ gtk_paned_set_position (GtkPaned *paned,
 
 #ifdef G_OS_WIN32
   /* Hacky work-around for bug #144269 */
-  gtk_widget_queue_resize (paned->child2);
+  if (paned->child2 != NULL)
+    {
+      gtk_widget_queue_draw (paned->child2);
+    }
 #endif
 }
 
@@ -1600,7 +1605,7 @@ get_child_panes (GtkWidget  *widget,
     }
   else if (GTK_IS_CONTAINER (widget))
     {
-      gtk_container_foreach (GTK_CONTAINER (widget),
+      gtk_container_forall (GTK_CONTAINER (widget),
 			     (GtkCallback)get_child_panes, panes);
     }
 }
