@@ -24,14 +24,16 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#ifndef GTK_TEXT_CHILD_H
-#define GTK_TEXT_CHILD_H
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
+#ifndef __GTK_TEXT_CHILD_H__
+#define __GTK_TEXT_CHILD_H__
 
 #include <glib-object.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 /* A GtkTextChildAnchor is a spot in the buffer where child widgets
  * can be "anchored" (inserted inline, as if they were characters).
@@ -60,6 +62,11 @@ struct _GtkTextChildAnchorClass
 {
   GObjectClass parent_class;
 
+  /* Padding for future expansion */
+  void (*_gtk_reserved1) (void);
+  void (*_gtk_reserved2) (void);
+  void (*_gtk_reserved3) (void);
+  void (*_gtk_reserved4) (void);
 };
 
 GType gtk_text_child_anchor_get_type (void) G_GNUC_CONST;
@@ -69,8 +76,6 @@ GtkTextChildAnchor* gtk_text_child_anchor_new (void);
 GList*   gtk_text_child_anchor_get_widgets (GtkTextChildAnchor *anchor);
 gboolean gtk_text_child_anchor_get_deleted (GtkTextChildAnchor *anchor);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif
