@@ -23,7 +23,7 @@
 #include <config.h>
 #include "gtktoolitem.h"
 #include "gtkmarshalers.h"
-#include "gtktoolbar.h"
+#include "gtktoolshell.h"
 #include "gtkseparatormenuitem.h"
 #include "gtkintl.h"
 #include "gtkmain.h"
@@ -32,6 +32,29 @@
 
 #include <string.h>
 
+/**
+ * SECTION:gtktoolitem
+ * @short_description: The base class of widgets that can be added to #GtkToolShell
+ *
+ * #GtkToolItem<!-- -->s are widgets that can appear on a toolbar. To
+ * create a toolbar item that contain something else than a button, use
+ * gtk_tool_item_new(). Use gtk_container_add() to add a child
+ * widget to the tool item.
+ *
+ * For toolbar items that contain buttons, see the #GtkToolButton,
+ * #GtkToggleToolButton and #GtkRadioToolButton classes.
+ *
+ * See the #GtkToolbar class for a description of the toolbar widget, and
+ * #GtkToolShell for a description of the tool shell interface.
+ */
+
+/**
+ * GtkToolItem:
+ *
+ * The GtkToolItem struct contains only private data.
+ * It should only be accessed through the functions described below.
+ */
+
 enum {
   CREATE_MENU_PROXY,
   TOOLBAR_RECONFIGURED,
@@ -148,13 +171,13 @@ gtk_tool_item_class_init (GtkToolItemClass *klass)
 
 /**
  * GtkToolItem::create-menu-proxy:
- * @toolitem: the object the signal was emitted on
+ * @tool_item: the object the signal was emitted on
  *
  * This signal is emitted when the toolbar needs information from @tool_item
  * about whether the item should appear in the toolbar overflow menu. In
  * response the tool item should either
  * <itemizedlist>
- * <listitem> call gtk_tool_item_set_proxy_menu_item() with a %NULL
+ * <listitem>call gtk_tool_item_set_proxy_menu_item() with a %NULL
  * pointer and return %TRUE to indicate that the item should not appear
  * in the overflow menu
  * </listitem>
@@ -186,16 +209,16 @@ gtk_tool_item_class_init (GtkToolItemClass *klass)
 
 /**
  * GtkToolItem::toolbar-reconfigured:
- * @toolitem: the object the signal was emitted on
+ * @tool_item: the object the signal was emitted on
  *
  * This signal is emitted when some property of the toolbar that the
  * item is a child of changes. For custom subclasses of #GtkToolItem,
  * the default handler of this signal use the functions
  * <itemizedlist>
- * <listitem>gtk_toolbar_get_orientation()</listitem>
- * <listitem>gtk_toolbar_get_style()</listitem>
- * <listitem>gtk_toolbar_get_icon_size()</listitem>
- * <listitem>gtk_toolbar_get_relief_style()</listitem>
+ * <listitem>gtk_tool_shell_get_orientation()</listitem>
+ * <listitem>gtk_tool_shell_get_style()</listitem>
+ * <listitem>gtk_tool_shell_get_icon_size()</listitem>
+ * <listitem>gtk_tool_shell_get_relief_style()</listitem>
  * </itemizedlist>
  * to find out what the toolbar should look like and change
  * themselves accordingly.
@@ -210,7 +233,7 @@ gtk_tool_item_class_init (GtkToolItemClass *klass)
 		  G_TYPE_NONE, 0);
 /**
  * GtkToolItem::set-tooltip:
- * @toolitem: the object the signal was emitted on
+ * @tool_item: the object the signal was emitted on
  * @tooltips: the #GtkTooltips
  * @tip_text: the tooltip text
  * @tip_private: the tooltip private text
@@ -220,6 +243,9 @@ gtk_tool_item_class_init (GtkToolItemClass *klass)
  * set the item's tooltip.
  *
  * Return value: %TRUE if the signal was handled, %FALSE if not
+ *
+ * Deprecated: 2.12: With the new tooltip API, there is no
+ *   need to use this signal anymore.
  **/
   toolitem_signals[SET_TOOLTIP] =
     g_signal_new (I_("set_tooltip"),
@@ -268,7 +294,7 @@ gtk_tool_item_parent_set (GtkWidget   *toolitem,
 			  GtkWidget   *prev_parent)
 {
   if (GTK_WIDGET (toolitem)->parent != NULL)
-    _gtk_tool_item_toolbar_reconfigured (GTK_TOOL_ITEM (toolitem));
+    gtk_tool_item_toolbar_reconfigured (GTK_TOOL_ITEM (toolitem));
 }
 
 static void
@@ -292,6 +318,7 @@ gtk_tool_item_set_property (GObject      *object,
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
     }
 }
 
@@ -316,6 +343,7 @@ gtk_tool_item_get_property (GObject    *object,
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
     }
 }
 
@@ -515,10 +543,10 @@ gtk_tool_item_get_icon_size (GtkToolItem *tool_item)
   g_return_val_if_fail (GTK_IS_TOOL_ITEM (tool_item), GTK_ICON_SIZE_LARGE_TOOLBAR);
 
   parent = GTK_WIDGET (tool_item)->parent;
-  if (!parent || !GTK_IS_TOOLBAR (parent))
+  if (!parent || !GTK_IS_TOOL_SHELL (parent))
     return GTK_ICON_SIZE_LARGE_TOOLBAR;
 
-  return gtk_toolbar_get_icon_size (GTK_TOOLBAR (parent));
+  return gtk_tool_shell_get_icon_size (GTK_TOOL_SHELL (parent));
 }
 
 /**
@@ -542,10 +570,10 @@ gtk_tool_item_get_orientation (GtkToolItem *tool_item)
   g_return_val_if_fail (GTK_IS_TOOL_ITEM (tool_item), GTK_ORIENTATION_HORIZONTAL);
 
   parent = GTK_WIDGET (tool_item)->parent;
-  if (!parent || !GTK_IS_TOOLBAR (parent))
+  if (!parent || !GTK_IS_TOOL_SHELL (parent))
     return GTK_ORIENTATION_HORIZONTAL;
 
-  return gtk_toolbar_get_orientation (GTK_TOOLBAR (parent));
+  return gtk_tool_shell_get_orientation (GTK_TOOL_SHELL (parent));
 }
 
 /**
@@ -585,10 +613,10 @@ gtk_tool_item_get_toolbar_style (GtkToolItem *tool_item)
   g_return_val_if_fail (GTK_IS_TOOL_ITEM (tool_item), GTK_TOOLBAR_ICONS);
 
   parent = GTK_WIDGET (tool_item)->parent;
-  if (!parent || !GTK_IS_TOOLBAR (parent))
+  if (!parent || !GTK_IS_TOOL_SHELL (parent))
     return GTK_TOOLBAR_ICONS;
 
-  return gtk_toolbar_get_style (GTK_TOOLBAR (parent));
+  return gtk_tool_shell_get_style (GTK_TOOL_SHELL (parent));
 }
 
 /**
@@ -613,10 +641,10 @@ gtk_tool_item_get_relief_style (GtkToolItem *tool_item)
   g_return_val_if_fail (GTK_IS_TOOL_ITEM (tool_item), GTK_RELIEF_NONE);
 
   parent = GTK_WIDGET (tool_item)->parent;
-  if (!parent || !GTK_IS_TOOLBAR (parent))
+  if (!parent || !GTK_IS_TOOL_SHELL (parent))
     return GTK_RELIEF_NONE;
 
-  return gtk_toolbar_get_relief_style (GTK_TOOLBAR (parent));
+  return gtk_tool_shell_get_relief_style (GTK_TOOL_SHELL (parent));
 }
 
 /**
@@ -773,7 +801,7 @@ gtk_tool_item_real_set_tooltip (GtkToolItem *tool_item,
   if (!child)
     return FALSE;
 
-  gtk_tooltips_set_tip (tooltips, child, tip_text, tip_private);
+  gtk_widget_set_tooltip_text (child, tip_text);
 
   return TRUE;
 }
@@ -790,6 +818,8 @@ gtk_tool_item_real_set_tooltip (GtkToolItem *tool_item,
  * to be used. See gtk_tooltips_set_tip().
  * 
  * Since: 2.4
+ *
+ * Deprecated: 2.12: Use gtk_tool_item_set_tooltip_text() instead.
  **/
 void
 gtk_tool_item_set_tooltip (GtkToolItem *tool_item,
@@ -805,15 +835,63 @@ gtk_tool_item_set_tooltip (GtkToolItem *tool_item,
 		 tooltips, tip_text, tip_private, &retval);
 }
 
+/**
+ * gtk_tool_item_set_tooltip_text:
+ * @tool_item: a #GtkToolItem: 
+ * @text: text to be used as tooltip for @tool_item
+ *
+ * Sets the text to be displayed as tooltip on the item.
+ * See gtk_widget_set_tooltip_text().
+ *
+ * Since: 2.12
+ **/
+void
+gtk_tool_item_set_tooltip_text (GtkToolItem *tool_item,
+			        const gchar *text)
+{
+  GtkWidget *child;
+
+  g_return_if_fail (GTK_IS_TOOL_ITEM (tool_item));
+
+  child = GTK_BIN (tool_item)->child;
+
+  if (child)
+    gtk_widget_set_tooltip_text (child, text);
+}
+
+/**
+ * gtk_tool_item_set_tooltip_markup:
+ * @tool_item: a #GtkToolItem: 
+ * @markup: markup text to be used as tooltip for @tool_item
+ *
+ * Sets the markup text to be displayed as tooltip on the item.
+ * See gtk_widget_set_tooltip_markup().
+ *
+ * Since: 2.12
+ **/
+void
+gtk_tool_item_set_tooltip_markup (GtkToolItem *tool_item,
+				  const gchar *markup)
+{
+  GtkWidget *child;
+
+  g_return_if_fail (GTK_IS_TOOL_ITEM (tool_item));
+
+  child = GTK_BIN (tool_item)->child;
+
+  if (child)
+    gtk_widget_set_tooltip_markup (child, markup);
+}
+
 /**
  * gtk_tool_item_set_use_drag_window:
- * @toolitem: a #GtkToolItem 
- * @use_drag_window: Whether @toolitem has a drag window.
+ * @tool_item: a #GtkToolItem 
+ * @use_drag_window: Whether @tool_item has a drag window.
  * 
- * Sets whether @toolitem has a drag window. When %TRUE the
+ * Sets whether @tool_item has a drag window. When %TRUE the
  * toolitem can be used as a drag source through gtk_drag_source_set().
- * When @toolitem has a drag window it will intercept all events,
- * even those that would otherwise be sent to a child of @toolitem.
+ * When @tool_item has a drag window it will intercept all events,
+ * even those that would otherwise be sent to a child of @tool_item.
  * 
  * Since: 2.4
  **/
@@ -847,12 +925,12 @@ gtk_tool_item_set_use_drag_window (GtkToolItem *toolitem,
 
 /**
  * gtk_tool_item_get_use_drag_window:
- * @toolitem: a #GtkToolItem 
+ * @tool_item: a #GtkToolItem 
  * 
- * Returns whether @toolitem has a drag window. See
+ * Returns whether @tool_item has a drag window. See
  * gtk_tool_item_set_use_drag_window().
  * 
- * Return value: %TRUE if @toolitem uses a drag window.
+ * Return value: %TRUE if @tool_item uses a drag window.
  * 
  * Since: 2.4
  **/
@@ -866,10 +944,10 @@ gtk_tool_item_get_use_drag_window (GtkToolItem *toolitem)
 
 /**
  * gtk_tool_item_set_visible_horizontal:
- * @toolitem: a #GtkToolItem
- * @visible_horizontal: Whether @toolitem is visible when in horizontal mode
+ * @tool_item: a #GtkToolItem
+ * @visible_horizontal: Whether @tool_item is visible when in horizontal mode
  * 
- * Sets whether @toolitem is visible when the toolbar is docked horizontally.
+ * Sets whether @tool_item is visible when the toolbar is docked horizontally.
  * 
  * Since: 2.4
  **/
@@ -893,12 +971,12 @@ gtk_tool_item_set_visible_horizontal (GtkToolItem *toolitem,
 
 /**
  * gtk_tool_item_get_visible_horizontal:
- * @toolitem: a #GtkToolItem 
+ * @tool_item: a #GtkToolItem 
  * 
- * Returns whether the @toolitem is visible on toolbars that are
+ * Returns whether the @tool_item is visible on toolbars that are
  * docked horizontally.
  * 
- * Return value: %TRUE if @toolitem is visible on toolbars that are
+ * Return value: %TRUE if @tool_item is visible on toolbars that are
  * docked horizontally.
  * 
  * Since: 2.4
@@ -913,14 +991,14 @@ gtk_tool_item_get_visible_horizontal (GtkToolItem *toolitem)
 
 /**
  * gtk_tool_item_set_visible_vertical:
- * @toolitem: a #GtkToolItem 
- * @visible_vertical: whether @toolitem is visible when the toolbar
+ * @tool_item: a #GtkToolItem 
+ * @visible_vertical: whether @tool_item is visible when the toolbar
  * is in vertical mode
  *
- * Sets whether @toolitem is visible when the toolbar is docked
+ * Sets whether @tool_item is visible when the toolbar is docked
  * vertically. Some tool items, such as text entries, are too wide to be
  * useful on a vertically docked toolbar. If @visible_vertical is %FALSE
- * @toolitem will not appear on toolbars that are docked vertically.
+ * @tool_item will not appear on toolbars that are docked vertically.
  * 
  * Since: 2.4
  **/
@@ -944,12 +1022,12 @@ gtk_tool_item_set_visible_vertical (GtkToolItem *toolitem,
 
 /**
  * gtk_tool_item_get_visible_vertical:
- * @toolitem: a #GtkToolItem 
+ * @tool_item: a #GtkToolItem 
  * 
- * Returns whether @toolitem is visible when the toolbar is docked vertically.
+ * Returns whether @tool_item is visible when the toolbar is docked vertically.
  * See gtk_tool_item_set_visible_vertical().
  * 
- * Return value: Whether @toolitem is visible when the toolbar is docked vertically
+ * Return value: Whether @tool_item is visible when the toolbar is docked vertically
  * 
  * Since: 2.4
  **/
@@ -1041,9 +1119,9 @@ gtk_tool_item_rebuild_menu (GtkToolItem *tool_item)
   g_return_if_fail (GTK_IS_TOOL_ITEM (tool_item));
 
   widget = GTK_WIDGET (tool_item);
-  
-  if (widget->parent && GTK_IS_TOOLBAR (widget->parent))
-    _gtk_toolbar_rebuild_menu (GTK_TOOLBAR (widget->parent));
+
+  if (GTK_IS_TOOL_SHELL (widget->parent))
+    gtk_tool_shell_rebuild_menu (GTK_TOOL_SHELL (widget->parent));
 }
 
 /**
@@ -1089,18 +1167,25 @@ gtk_tool_item_set_proxy_menu_item (GtkToolItem *tool_item,
 }
 
 /**
- * _gtk_tool_item_toolbar_reconfigured:
- * @tool_item: a #GtkToolItem: 
- * 
- * Emits the signal #GtkToolItem::toolbar_reconfigured on @tool_item. This
- * internal function is called by #GtkToolbar when some aspect of its
- * configuration changes.
- * 
- * Since: 2.4
+ * gtk_tool_item_toolbar_reconfigured:
+ * @tool_item: a #GtkToolItem
+ *
+ * Emits the signal #GtkToolItem::toolbar_reconfigured on @tool_item.
+ * #GtkToolbar and other #GtkToolShell implementations use this function
+ * to notify children, when some aspect of their configuration changes.
+ *
+ * Since: 2.14
  **/
 void
-_gtk_tool_item_toolbar_reconfigured (GtkToolItem *tool_item)
+gtk_tool_item_toolbar_reconfigured (GtkToolItem *tool_item)
 {
+  /* The slightely inaccurate name "gtk_tool_item_toolbar_reconfigured" was
+   * choosen over "gtk_tool_item_tool_shell_reconfigured", since the function
+   * emits the "toolbar-reconfigured" signal, not "tool-shell-reconfigured".
+   * Its not possible to rename the signal, and emitting another name than
+   * indicated by the function name would be quite confusing. That's the
+   * price of providing stable APIs.
+   */
   g_return_if_fail (GTK_IS_TOOL_ITEM (tool_item));
 
   g_signal_emit (tool_item, toolitem_signals[TOOLBAR_RECONFIGURED], 0);
