@@ -17,6 +17,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_TREE_STORE_H__
 #define __GTK_TREE_STORE_H__
 
@@ -86,6 +90,11 @@ void          gtk_tree_store_set_value        (GtkTreeStore *tree_store,
 void          gtk_tree_store_set              (GtkTreeStore *tree_store,
 					       GtkTreeIter  *iter,
 					       ...);
+void          gtk_tree_store_set_valuesv      (GtkTreeStore *tree_store,
+					       GtkTreeIter  *iter,
+					       gint         *columns,
+					       GValue       *values,
+					       gint          n_values);
 void          gtk_tree_store_set_valist       (GtkTreeStore *tree_store,
 					       GtkTreeIter  *iter,
 					       va_list       var_args);
@@ -103,6 +112,18 @@ void          gtk_tree_store_insert_after     (GtkTreeStore *tree_store,
 					       GtkTreeIter  *iter,
 					       GtkTreeIter  *parent,
 					       GtkTreeIter  *sibling);
+void          gtk_tree_store_insert_with_values (GtkTreeStore *tree_store,
+						 GtkTreeIter  *iter,
+						 GtkTreeIter  *parent,
+						 gint          position,
+						 ...);
+void          gtk_tree_store_insert_with_valuesv (GtkTreeStore *tree_store,
+						  GtkTreeIter  *iter,
+						  GtkTreeIter  *parent,
+						  gint          position,
+						  gint         *columns,
+						  GValue       *values,
+						  gint          n_values);
 void          gtk_tree_store_prepend          (GtkTreeStore *tree_store,
 					       GtkTreeIter  *iter,
 					       GtkTreeIter  *parent);
