@@ -33,12 +33,11 @@
 
 #include <glib/gi18n-lib.h>
 
-#include "gtkdebug.h"
-#include "gtkprintoperation.h"
-#include "gtkprintbackendlpr.h"
-#include "gtkprinter.h"
+#include <gtk/gtk.h>
 #include "gtkprinter-private.h"
 
+#include "gtkprintbackendlpr.h"
+
 typedef struct _GtkPrintBackendLprClass GtkPrintBackendLprClass;
 
 #define GTK_PRINT_BACKEND_LPR_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_PRINT_BACKEND_LPR, GtkPrintBackendLprClass))
